@@ -17,8 +17,8 @@ def handle(data, secrets, function_call_info):
       output = service(data, secrets)
       cdf.api_keys = secrets
       cdf.set_up_cdf_client(context='deploy')
-      metadata = cdf.get_function_metadata(function_call_info)
-      output.update(metadata)
+      metadata = cdf.get_function_metadata(function_call_info["function_id"])
+      output.update(dict(metadata=metadata))
       return output
    except Exception:
       trace = traceback.format_exc()
