@@ -9,6 +9,7 @@ import time
 import sys
 import os 
 import json
+import functools
 from shutil import make_archive, unpack_archive
 
 from cognite.client.exceptions import CogniteNotFoundError, CogniteAPIError
@@ -75,7 +76,7 @@ def get_client(api_key, api_key_label=None):
     return client
 
 
-def create_function(
+def create_function_from_folder(
     function_name, 
     folder, 
     handler_path,  
@@ -117,35 +118,54 @@ def create_function(
     return function
 
 
-def deploy_function(
-    function_name,
-    folder='.',
-    handler_path='handler.py',
-    secrets=api_keys,
-    info={'description':'', 'owner':''}
+def create_function_from_file(
+    function_name, 
+    file_id, 
+    owner,
+    description,
+    secrets={}
 ):
     """
-    Deploys a Cognite function from a folder. The function is created first,
-    and then it waits until the function status is ready or failed. If it fails,
-    it will try again `max_error` times
+    Create a Cognite function from a file deployed to CDF Files. CDF raises
+    exception if there's an existing function with the same name.
 
     Inputs:
-      - function_name: name of the function to create
-      - folder: path where the source code is located
-      - handler_path: path to the handler file
-      - secrets: api keys or similar that should be passed to the function
+      - function_name: (string) name of the function to create
+      - file_id: (string) the id for the function file in CDF Files
+      - owner: (string) the function's owner's email
+      - secrets: (optional) api keys or similar that should be passed to the
+        function
+    """
+    client = global_client["functions"]
+    
+    function = client.functions.create( 
+        name=function_name, 
+        file_id=file_id,
+        external_id=function_name,
+        description=description,
+        owner=owner,
+        secrets=secrets,
+        env_vars=env_vars
+    )
+    logging.debug(
+        f"Created {function_name}: {file_id=}, {env_vars=}"
+    )
+
+    return function
+
+
+def robust_create(create_function):
+    """
+    Robust creation of a CDF Function. Wait until the function status is ready
+    or failed. If it fails, it will try again `max_error` times
+
+    Inputs:
+      - create_function: function that creates the CDF function
     """
     max_errors = 3
 
     for trial in range(max_errors):
-        function = create_function(
-            function_name, 
-            folder, 
-            handler_path, 
-            info['description'],
-            info['owner'],
-            secrets
-        )
+        function = create_function()
         status = wait_function_status(function)
         logging.debug(f"Function status is {status}")
         if function.status == 'Ready':
@@ -158,9 +178,65 @@ def deploy_function(
             raise Exception(f"Function deployment error: {function.error=}")
 
 
-def get_function_metadata(function_call_info):
+def deploy_function(
+    function_name,
+    folder='.',
+    handler_path='handler.py',
+    secrets=api_keys,
+    info={'description':'', 'owner':''}
+):
+    """
+    Deploys a Cognite function from a folder. 
+
+    Inputs:
+      - function_name: name of the function to create
+      - folder: path where the source code is located
+      - handler_path: path to the handler file
+      - secrets: api keys or similar that should be passed to the function
+    """
+    f = functools.partial(
+        create_function_from_folder,
+        function_name, 
+        folder, 
+        handler_path, 
+        info['description'],
+        info['owner'],
+        secrets
+    )
+    robust_create(f)
+ 
+
+def redeploy_function(
+    function_name,
+    file_id,
+    description,
+    owner,
+    secrets=api_keys
+):
+    """
+    Deploys a Cognite function from a folder. 
+
+    Inputs:
+      - function_name: name of the function to create
+      - file_id: (string) the id for the function file in CDF Files
+      - owner: (string) the function's owner's email
+      - secrets: (optional) api keys or similar that should be passed to the
+        function
+    """
+    f = functools.partial(
+        create_function_from_file,
+        function_name, 
+        file_id, 
+        description,
+        owner,
+        secrets
+    )
+    robust_create(f)      
+
+
+def get_function_metadata(function_id):
+   
    client = global_client["functions"]
-   function_id=function_call_info["function_id"]
    function = client.functions.retrieve(id=function_id)
    metadata = dict(
       external_id=function.external_id,
